from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings import Namespace

namespace = Namespace(name='exif', label=_('EXIF'))
setting_exif_backend = namespace.add_setting(
    global_name='EXIF_BACKEND', default='exif.backends.exiftool.EXIFTool',
    help_text=_(
        'Full path to the backend to be used to extract the EXIF data.'
    )
)



from __future__ import unicode_literals

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _


class DocumentVersionEXIFHelper(object):
    @staticmethod
    @property
    def constructor(source_object):
        return DocumentVersionEXIFHelper(source_object)

    def __init__(self, instance):
        self.instance = instance

    def __getattr__(self, tag):
        try:
            return self.instance.exif.get(tag=tag).value
        except ObjectDoesNotExist:
            raise AttributeError(
                _('EXIF Tag \'%s\' not found') % tag
            )


class DocumentEXIFHelper(DocumentVersionEXIFHelper):
    @staticmethod
    @property
    def constructor(source_object):
        return DocumentEXIFHelper(source_object)

    def __init__(self, instance):
        self.instance = instance.latest_version



from __future__ import unicode_literals

import json
import logging
import tempfile

import sh

from common.utils import fs_cleanup

from .literals import EXIFTOOL_PATH

try:
    exiftool = sh.Command(EXIFTOOL_PATH)
except sh.CommandNotFound:
    exiftool = None
else:
    exiftool = exiftool.bake('-j')

logger = logging.getLogger(__name__)


class EXIFTool(object):
    def execute(self, document_version):
        new_file_object, temp_filename = tempfile.mkstemp()

        try:
            document_version.save_to_file(filepath=temp_filename)
            result = exiftool(temp_filename)
            return json.loads(result.stdout)[0]
        finally:
            fs_cleanup(filename=temp_filename)



from __future__ import unicode_literals

from django.core.files import File
from django.test import override_settings

from common.tests.test_views import GenericViewTestCase
from documents.models import DocumentType
from documents.permissions import permission_document_view
from documents.tests import TEST_DOCUMENT_TYPE
from user_management.tests.literals import (
    TEST_USER_PASSWORD, TEST_USER_USERNAME
)

from .literals import TEST_SMALL_DOCUMENT_PATH


@override_settings(OCR_AUTO_OCR=False)
class DocumentsVersionEXIFViewsTestCase(GenericViewTestCase):
    def setUp(self):
        super(DocumentsVersionEXIFViewsTestCase, self).setUp()
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        super(DocumentsVersionEXIFViewsTestCase, self).tearDown()
        if self.document_type.pk:
            self.document_type.delete()

    def test_document_version_exit_view_no_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        response = self.get(
            'exif:document_version_exif_list',
            args=(self.document.latest_version.pk,)
        )
        self.assertEqual(response.status_code, 403)

    def test_document_version_exif_view_with_permission(self):
        self.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )

        self.role.permissions.add(
            permission_document_view.stored_permission
        )
        response = self.get(
            'exif:document_version_exif_list',
            args=(self.document.latest_version.pk,), follow=True
        )
        self.assertContains(response, text='FileType', status_code=200)



from __future__ import unicode_literals

import logging

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from documents.models import DocumentVersion

from .managers import DocumentVersionEXIFDataManager

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class DocumentVersionEXIFData(models.Model):
    """
    Model to store an EXIF tag and value for a document version.
    """

    document_version = models.ForeignKey(
        DocumentVersion, related_name='exif',
        verbose_name=_('Document version')
    )

    tag = models.CharField(max_length=255, verbose_name=_('Tag'))
    value = models.CharField(max_length=255, verbose_name=_('Value'))

    objects = DocumentVersionEXIFDataManager()

    class Meta:
        verbose_name = _('Document version EXIF data')
        verbose_name_plural = _('Document versions EXIF data')

    def __str__(self):
        return '{}: {}'.format(self.tag, self.value)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0028_newversionblock'),
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentVersionEXIFData',
            fields=[
                (
                    'id', models.AutoField(
                        verbose_name='ID', serialize=False, auto_created=True,
                        primary_key=True
                    )
                ),
                (
                    'tag', models.CharField(
                        max_length=255, verbose_name='Tag'
                    )
                ),
                (
                    'value', models.CharField(
                        max_length=255, verbose_name='Value'
                    )
                ),
                (
                    'document_version', models.ForeignKey(
                        related_name='exif', verbose_name='Document version',
                        to='documents.DocumentVersion'
                    )
                ),
            ],
            options={
                'verbose_name': 'Document version EXIF data',
                'verbose_name_plural': 'Document versions EXIF data',
            },
            bases=(models.Model,),
        ),
    ]



from __future__ import unicode_literals

from __future__ import unicode_literals

__author__ = 'Roberto Rosario'
__build__ = 0x010000
__copyright__ = 'Copyright 2015 Roberto Rosario'
__license__ = 'MIT'
__title__ = 'mayan-exif'
__version__ = '1.0.0'

default_app_config = 'exif.apps.EXIFApp'



from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import DocumentVersionEXIFListView

urlpatterns = patterns(
    '',
    url(
        r'^document_version/(?P<pk>\d+)/exif/$',
        DocumentVersionEXIFListView.as_view(),
        name='document_version_exif_list'
    ),
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from documents.permissions import permission_document_view
from navigation import Link

link_document_version_exif_list = Link(
    permissions=(permission_document_view,), text=_('EXIF'),
    view='exif:document_version_exif_list', args='object.id'
)



from __future__ import absolute_import, unicode_literals

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext_lazy as _

from acls.models import AccessControlList
from common.views import SingleObjectListView
from documents.models import DocumentVersion
from documents.permissions import permission_document_view
from permissions import Permission


class DocumentVersionEXIFListView(SingleObjectListView):
    def get_extra_context(self):
        return {
            'hide_object': True,
            'object': self.get_object(),
            'document': self.get_object().document,
            'navigation_object_list': ('object', 'document'),
            'title': _('EXIF data for: %s') % self.get_object(),
        }

    def get_object(self):
        document_version = get_object_or_404(
            DocumentVersion, pk=self.kwargs['pk']
        )

        try:
            Permission.check_permissions(
                self.request.user, (permission_document_view,)
            )
        except PermissionDenied:
            AccessControlList.objects.check_access(
                permission_document_view, self.request.user, document_version
            )

        return document_version

    def get_queryset(self):
        return self.get_object().exif.all()



from __future__ import unicode_literals

import os

from documents.tests import TEST_SMALL_DOCUMENT_FILENAME

TEST_SMALL_DOCUMENT_PATH = os.path.join(
    'contrib', 'sample_documents',
    TEST_SMALL_DOCUMENT_FILENAME
)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from common import menu_object
from common.apps import MayanAppConfig
from documents.models import Document, DocumentVersion
from documents.signals import post_version_upload

from navigation import SourceColumn

from .classes import DocumentEXIFHelper, DocumentVersionEXIFHelper
from .handlers import extract_exif
from .links import link_document_version_exif_list
from .models import DocumentVersionEXIFData


class EXIFApp(MayanAppConfig):
    name = 'exif'
    test = True
    verbose_name = _('Document EXIF data')

    def ready(self):
        super(EXIFApp, self).ready()

        Document.add_to_class(
            'exif_value_of', DocumentEXIFHelper.constructor
        )

        DocumentVersion.add_to_class(
            'exif_value_of', DocumentVersionEXIFHelper.constructor
        )

        SourceColumn(
            source=DocumentVersionEXIFData, label=_('Tag'),
            attribute='tag'
        )

        SourceColumn(
            source=DocumentVersionEXIFData, label=_('Value'),
            attribute='value'
        )

        menu_object.bind_links(
            links=(link_document_version_exif_list,),
            sources=(DocumentVersion,)
        )

        # Extract EXIF data of each newly created document version
        post_version_upload.connect(
            extract_exif, dispatch_uid='extract_exif',
            sender=DocumentVersion
        )



from __future__ import unicode_literals

from .models import DocumentVersionEXIFData


def extract_exif(sender, instance, **kwargs):
    """Extract EXIF data of each newly created document version"""
    DocumentVersionEXIFData.objects.process_version(document_version=instance)



from __future__ import unicode_literals

import logging

from django.db import models
from django.utils.module_loading import import_string

from .settings import setting_exif_backend

logger = logging.getLogger(__name__)


class DocumentVersionEXIFDataManager(models.Manager):
    def process_version(self, document_version):
        backend_class = import_string(setting_exif_backend.value)
        backend = backend_class()

        for tag, value in backend.execute(document_version=document_version).items():
            self.create(
                document_version=document_version, tag=tag, value=value
            )



from __future__ import unicode_literals

from django.core.files.base import File
from django.test import TestCase, override_settings

from documents.models import DocumentType
from documents.tests import TEST_DOCUMENT_TYPE

from ..models import DocumentVersionEXIFData

from .literals import TEST_SMALL_DOCUMENT_PATH


@override_settings(OCR_AUTO_OCR=False)
class DocumentVersionEXIFDataTestCase(TestCase):
    def setUp(self):
        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()

    def test_handler(self):
        exif_data = DocumentVersionEXIFData.objects.get(
            document_version=self.document.latest_version, tag='FileType'
        )

        self.assertEqual(exif_data.value, 'PNG')

    def test_model_str_method(self):
        self.assertEqual(
            unicode(self.document.latest_version.exif.get(tag='FileType')),
            'FileType: PNG'
        )
